"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvmNtt = exports.EvmNttWormholeTranceiver = exports.evmNttProtocolFactory = void 0;
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
const sdk_evm_1 = require("@wormhole-foundation/sdk-evm");
require("@wormhole-foundation/sdk-evm-core");
const index_js_1 = require("./index.js");
function evmNttProtocolFactory(token) {
    class _EvmNtt extends EvmNtt {
        tokenAddress = token;
        static async fromRpc(provider, config) {
            const [network, chain] = await sdk_evm_1.EvmPlatform.chainFromRpc(provider);
            const conf = config[chain];
            if (conf.network !== network)
                throw new Error(`Network mismatch: ${conf.network} != ${network}`);
            if (!conf.tokenMap)
                throw new Error('Token map not found');
            const maybeToken = sdk_connect_1.tokens.filters.byAddress(conf.tokenMap, token);
            if (maybeToken === undefined)
                throw new Error('Token not found');
            if (!maybeToken.ntt)
                throw new Error('Token not configured with NTT');
            const { manager, transceiver } = maybeToken.ntt;
            return new _EvmNtt(network, chain, provider, {
                manager,
                transceiver: { wormhole: transceiver },
            });
        }
    }
    return _EvmNtt;
}
exports.evmNttProtocolFactory = evmNttProtocolFactory;
class EvmNttWormholeTranceiver {
    manager;
    address;
    transceiver;
    constructor(manager, address) {
        this.manager = manager;
        this.address = address;
        this.transceiver =
            index_js_1.ethers_contracts.factories.WormholeTransceiver__factory.connect(address, manager.provider);
    }
    encodeFlags(skipRelay) {
        return new Uint8Array([skipRelay ? 1 : 0]);
    }
    async *receive(attestation) {
        const tx = await this.transceiver.receiveMessage.populateTransaction((0, sdk_connect_1.serialize)(attestation));
        yield this.manager.createUnsignedTx(tx, 'WormholeTransceiver.receiveMessage');
    }
    async isWormholeRelayingEnabled(destChain) {
        return await this.transceiver.isWormholeRelayingEnabled((0, sdk_connect_1.toChainId)(destChain));
    }
    async isSpecialRelayingEnabled(destChain) {
        return await this.transceiver.isSpecialRelayingEnabled((0, sdk_connect_1.toChainId)(destChain));
    }
}
exports.EvmNttWormholeTranceiver = EvmNttWormholeTranceiver;
class EvmNtt {
    network;
    chain;
    provider;
    contracts;
    chainId;
    manager;
    xcvrs;
    managerAddress;
    constructor(network, chain, provider, contracts) {
        this.network = network;
        this.chain = chain;
        this.provider = provider;
        this.contracts = contracts;
        this.chainId = sdk_connect_1.nativeChainIds.networkChainToNativeChainId.get(network, chain);
        this.managerAddress = contracts.manager;
        this.manager = index_js_1.ethers_contracts.factories.NttManager__factory.connect(contracts.manager, this.provider);
        this.xcvrs = [
            // Enable more Transceivers here
            new EvmNttWormholeTranceiver(this, contracts.transceiver.wormhole),
        ];
    }
    encodeFlags(enabledIdxs) {
        return this.xcvrs
            .map((xcvr, idx) => {
            if (!enabledIdxs || enabledIdxs.includes(idx))
                return { index: idx, payload: xcvr.encodeFlags(true) };
            return null;
        })
            .filter((x) => x !== null);
    }
    async quoteDeliveryPrice(dstChain) {
        return this.manager.quoteDeliveryPrice.staticCall((0, sdk_connect_1.toChainId)(dstChain), this.encodeFlags(), this.xcvrs.map((x) => x.address));
    }
    async *transfer(sender, amount, destination, queue) {
        const [_, totalPrice] = await this.quoteDeliveryPrice(destination.chain);
        const transceiverIxs = sdk_connect_1.Ntt.encodeTransceiverInstructions(this.encodeFlags());
        const senderAddress = new sdk_evm_1.EvmAddress(sender).toString();
        //TODO check for ERC-2612 (permit) support on token?
        const tokenContract = sdk_evm_1.EvmPlatform.getTokenImplementation(this.provider, this.tokenAddress);
        const allowance = await tokenContract.allowance(senderAddress, this.managerAddress);
        if (allowance < amount) {
            const txReq = await tokenContract.approve.populateTransaction(this.managerAddress, amount);
            yield this.createUnsignedTx((0, sdk_evm_1.addFrom)(txReq, senderAddress), 'TokenBridge.Approve');
        }
        const txReq = await this.manager
            .getFunction('transfer(uint256,uint16,bytes32,bool,bytes)')
            .populateTransaction(amount, (0, sdk_connect_1.toChainId)(destination.chain), (0, sdk_connect_1.universalAddress)(destination), queue, transceiverIxs, { value: totalPrice });
        yield this.createUnsignedTx((0, sdk_evm_1.addFrom)(txReq, senderAddress), 'Ntt.transfer');
    }
    // TODO: should this be some map of idx to transceiver?
    async *redeem(attestations) {
        if (attestations.length !== this.xcvrs.length)
            throw 'no';
        for (const idx in this.xcvrs) {
            const xcvr = this.xcvrs[idx];
            yield* xcvr.receive(attestations[idx]);
        }
    }
    async getCurrentOutboundCapacity() {
        return await this.manager.getCurrentOutboundCapacity();
    }
    async getCurrentInboundCapacity(fromChain) {
        return await this.manager.getCurrentInboundCapacity((0, sdk_connect_1.toChainId)(fromChain));
    }
    async getRateLimitDuration() {
        return await this.manager.rateLimitDuration();
    }
    async getInboundQueuedTransfer(fromChain, transceiverMessage) {
        const queuedTransfer = await this.manager.getInboundQueuedTransfer(sdk_connect_1.Ntt.messageDigest(fromChain, transceiverMessage));
        if (queuedTransfer.txTimestamp > 0n) {
            const { recipient, amount, txTimestamp } = queuedTransfer;
            const duration = await this.getRateLimitDuration();
            return {
                recipient: new sdk_evm_1.EvmAddress(recipient),
                amount: amount,
                rateLimitExpiryTimestamp: Number(txTimestamp + duration),
            };
        }
        return null;
    }
    async *completeInboundQueuedTransfer(fromChain, transceiverMessage, token, payer) {
        const tx = await this.manager.completeInboundQueuedTransfer(sdk_connect_1.Ntt.messageDigest(fromChain, transceiverMessage));
        yield this.createUnsignedTx(tx, 'Ntt.completeInboundQueuedTransfer');
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new sdk_evm_1.EvmUnsignedTransaction((0, sdk_evm_1.addChainId)(txReq, this.chainId), this.network, this.chain, description, parallelizable);
    }
}
exports.EvmNtt = EvmNtt;
//# sourceMappingURL=ntt.js.map